function beta=HMMbackward(v,phghm,pvgh,thres)
%HMMBACKWARD HMM Backward Pass
% logbeta=HMMbackward(v,phghm,pvgh)
%
% Inputs:
% v : visible (observation) sequence being a vector v=[2 1 3 3 1 ...]
% phghm : homogeneous transition distribution phghm(i,j)=p(h(t)=i|h(t-1)=j)
% ph1 : initial distribution
% pvgh : homogeneous emission disrtribution pvgh(i,j)=p(v(t)=i|h(t)=j)
%
% Outputs:
% logbeta: log beta messages: log p(v(t+1:T)|h(t))
% See also HMMbackward.m, HMMviterbi.m, demoHMMinference.m
T=length(v); H=size(phghm,1);
phghmtrans=phghm'; pvghtrans=pvgh';
beta=sparse(zeros(H,T));
beta(:,T)=ones(H,1)./H;
for t=T:-1:2
    tmp=phghmtrans*(beta(:,t).*pvghtrans(:,v(t)));
    tmp2= bsxfun(@rdivide, tmp, sum(tmp));
    ind=(tmp2>thres);
    tmp3=tmp2(ind);
    beta(ind,t-1)= bsxfun(@rdivide, tmp3, sum(tmp3));
end