function [alpha loglik]=HMMforward(v,phghm,ph1,pvgh,thres)
%HMMFORWARD HMM Forward Pass
% [logalpha,loglik]=HMMforward(v,phghm,ph1,pvgh)
%
% Inputs:
% v : visible (observation) sequence being a vector v=[2 1 3 3 1 ...]
% phghm : homogeneous transition distribution phghm(i,j)=p(h(t)=i|h(t-1)=j)
% ph1 : initial distribution
% pvgh : homogeneous emission distribution pvgh(i,j)=p(v(t)=i|h(t)=j)
%
% Outputs:
% logalpha : log alpha messages: log p(h(t),v(1:t))
% loglik : sequence log likelihood log p(v(1:T))
% See also HMMbackward.m, HMMviterbi.m, demoHMMinference.m
T=length(v);
pvghtrans=pvgh';
alpha=sparse(zeros(length(ph1),T));
alpha(:,1) = pvgh(v(1),:)'.*ph1;
loglik=full(log(sum(alpha(:,1))));
for t=2:T
    tmp=pvghtrans(:,v(t)).*(phghm*alpha(:,t-1));
    tmp4=sum(tmp);
    loglik=loglik+full(log(tmp4));
    tmp2= bsxfun(@rdivide, tmp, tmp4);
    ind=(tmp2>thres);
    tmp3=tmp2(ind);
    alpha(ind,t)= bsxfun(@rdivide, tmp3, sum(tmp3));
end