function demoFirstname
% Search for *firstname*
% where * represents a random sequence
% To do this we need two general states and a set of pattern states

% Patterns in charcter form:
cpattern{1}='david';
cpattern{2}='anton';
cpattern{3}='fred';
cpattern{4}='jim';
cpattern{5}='barry';

for p=1:5
    pattern{p}=name2num(cpattern{p}); % convert to numerical states
end

firstname=1:5; % 5 first names
patternstate=1:5; % pattern states (1 per pattern)
generalstate=6:7; % general states

nstates=generalstate(end); % total number of pattern+general states

% make the generalised transistion matrix:
tran=zeros(nstates,nstates);
tran(generalstate(1),generalstate(1))=0.8; % not started firstname
for p=firstname
    tran(patternstate(p),generalstate(1))=0.2/length(firstname); % into a firstname
    tran(generalstate(2),patternstate(p))=1; % out of firstname
end
tran(generalstate(2),generalstate(2))=1;

prior.generalstate=[1 0 0]'; % start in general state 1 with certainty

%emission for general states:
for g=1:length(generalstate)
    pvgh_general(:,g)=ones(26,1)/26; % uniform emission for general states
end

% emission for patterns:

% one way to do this: fully flexible
if 1==1
    % emission for pattern states:
    for pt=patternstate
        for l=1:length(pattern{pt})
            a=0.9; % 90 percent prob of emitting the correct letter
            tmp=(1-a)/25*ones(26,1);
            tmp(pattern{pt}(l))=a;
            pvgh_pattern{pt,l}=tmp; % probability distribution for position l of pattern pt
        end
    end
    [phghm,pvgh,ph1,startpatternIDX,endpatternIDX,generalstateIDX]=patternsearchsetup(pattern,patternstate,generalstate,tran,pvgh_pattern,pvgh_general,prior);
else
    % Alternative simpler (less flexible) way using a corruption matrix:
    a=0.9; % 90 percent prob of emitting the correct letter
    CorruptionProb=ones(26,26)*(1-a)/25;
    for i=1:26
        CorruptionProb(i,i)=0.9;
    end
    [phghm,pvgh,ph1,startpatternIDX,endpatternIDX,generalstateIDX]=patternsearchsetup(pattern,patternstate,generalstate,tran,CorruptionProb,pvgh_general,prior);
end

% generate some data from the model

T=10; % length of observation sequence
[v,h]=HMMsample(T,phghm,pvgh,ph1);

fprintf(1,'observed sequence: %s\n',num2name(v))

thresh=0; % threshold for approximation. Set to zero for exact computation.
printtime=1;

[alpha,beta,gamma,delta,loglik,loglikpvhstar]=patternsearch(v,phghm,pvgh,ph1,thresh,printtime);

[pnum gnum]=getpattern(delta,startpatternIDX,generalstateIDX); % just get which patterns these correspond to.

fprintf(1,'\nViterbi sequence corresponds to:\n\n')

fprintf(1,'observation: [patternstate generalstate]')
for i=1:length(v)
    fprintf(1,'\n          %s:         [%d        %d]',num2name(v(i)),pnum(i),gnum(i));
    if pnum(i)>0
        fprintf(1,' %s',cpattern{pnum(i)})
    end
end
fprintf(1,'\n\nlikelihood log p(v_{1:T})=%f\n',loglik) % log probability of the sequence
fprintf(1,'likelihood log p(v_{1:T},h^*_{1:T})=%f\n',loglikpvhstar) % log probabilty of the joint seq and most likely explanation

